# MPENN - Massive Picture Editing for Neural Networks

MPENN is a simple software solution designed to transform the way we process and edit large volumes of images, particularly in applications involving neural networks. This versatile tool combines video-to-picture conversion capabilities with an advanced image editor, enabling users to manage and manipulate images efficiently at scale. Whether you are working on machine learning datasets, enhancing digital media assets, or require a robust tool for bulk image editing, MPENN is the ideal solution for your needs.

## Features

MPENN provides a comprehensive set of features designed to streamline the process of converting, editing, and managing large sets of images. Here's what you can expect:

- **Video Conversion**: Effortlessly convert videos into sequences of high-quality images, ready for further processing or analysis.

- **Fast Image Resizing**: Quickly adjust the sizes of your images to meet specific requirements without compromising on quality.

- **Customizable Bounding Boxes**: Annotate your images with bounding boxes and customize their appearance with various colors and thicknesses to suit your needs.

- **Data Tracking and Management**: For each output folder, MPENN generates a detailed `.json` file that contains important information about each image, including bounding box details. This helps you manage and analyze your data more efficiently.

With these features, MPENN significantly simplifies the workflow for users who need to process and analyze images at scale, especially those working with neural networks and AI applications.

# Getting Started with MPENN

Welcome to MPENN, a comprehensive tool designed to streamline massive picture editing tasks for neural networks. Follow these steps to get started with MPENN and begin transforming your videos and images into machine learning-ready datasets.

## 1. Download MPENN

To start using MPENN, first obtain the software by downloading it from its [Gitea repository](https://gitea.example.com/mpenn/mpenn). Click on the latest release suitable for your operating system to begin the download process.

## 2. Setting Up Your Environment

After downloading MPENN, it's recommended to set up a virtual environment (venv) for running the software. This ensures that your Python environment remains clean and organized. Open your terminal or command prompt and navigate to the MPENN directory. Then, run the following commands:

```bash
# Create a virtual environment named 'mpenn-env'
python -m venv mpenn-env

# Activate the virtual environment
# On Windows
mpenn-env\Scripts\activate
# On Unix or MacOS
source mpenn-env/bin/activate

# Install MPENN dependencies
pip install -r requirements.txt
```

## Running MPENN

With your environment set up, you're ready to launch MPENN. Run the `main.py` File to get started:

### 1. Converting Video to Picture Sequence

If you're starting with a video, MPENN makes it easy to convert it into a sequence of images. Use the video-to-picture feature to specify your video file and the output directory for the images.

### 2. Processing Image Sequences

Whether you've generated your images from a video or already have a sequence of images, you can start editing them with MPENN. You can:
    - Resize Images: Quickly adjust the resolution of your images to fit your requirements.
    - Create Bounding Boxes: Annotate your images with bounding boxes for object detection tasks.

## Documentation

In-depth documentation on using MPENN will be available soon.

## License

MPENN is made available under the [Attribution-NonCommercial-ShareAlike 4.0](https://creativecommons.org/licenses/by-nc-sa/4.0/) license. For more details, please refer to the [LICENCE](.../.././LICENSE) file included in the distribution.

## Connect with Me

Stay updated on the latest news and updates about MPENN by following us on our social media channels:

- [LinkedIn](https://www.linkedin.com/in/FalkoHabel/)